'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.reducer = reducer;
exports.push = push;
exports.update = update;
exports.updateIn = updateIn;
exports.remove = remove;
exports.removeIn = removeIn;

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _reduxActions = require('redux-actions');

var _utility = require('./utility');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Create a reducer
 */
function reducer(handlers, state) {
  return (0, _reduxActions.handleActions)(handlers, _immutable2.default.fromJS(state));
}

/**
 * Push item to state 
 */
function push(state, action) {
  (0, _utility.checkPayload)(action);

  var value = action.payload;
  return state.push(_immutable2.default.fromJS(value));
}

/**
 * Update a specific key or index
 */
function update(state, action) {
  (0, _utility.checkPayload)(action);
  (0, _utility.checkUpdate)(action);

  var _action$payload = action.payload;
  var key = _action$payload.key;
  var value = _action$payload.value;
  var fn = _action$payload.fn;


  if ((0, _utility.isUndefined)(key)) throw new Error('"key" must be present in payload!');

  return state.update(key, (0, _utility.modifier)(value, fn));
}

/**
 * Update at a selection path
 */
function updateIn(state, action) {
  (0, _utility.checkPayload)(action);
  (0, _utility.checkUpdate)(action);

  var _action$payload2 = action.payload;
  var selected = _action$payload2.selected;
  var value = _action$payload2.value;
  var fn = _action$payload2.fn;


  if ((0, _utility.isUndefined)(selected)) throw new Error('"selected" must be present in payload!');

  return state.updateIn(selected, (0, _utility.modifier)(value, fn));
}

/**
 * Remove a specific index or key
 */
function remove(state, action) {
  (0, _utility.checkPayload)(action);

  return state.remove(action.payload);
}

/**
 * Remove at a selection path
 */
function removeIn(state, action) {
  (0, _utility.checkPayload)(action);

  return state.removeIn(action.payload);
}